// D import file generated from 'tango/io/Path.d'
module tango.io.Path;
private import tango.sys.Common;

public import tango.time.Time : Time, TimeSpan;

private import tango.io.model.IFile : FileConst, FileInfo;

public import tango.core.Exception : IOException, IllegalArgumentException;

private import tango.stdc.string : memmove;

private import tango.core.Octal;

version (Win32)
{
	version (Win32SansUnicode)
	{
		private extern (C) int strlen(const char* s);


		private alias WIN32_FIND_DATA FIND_DATA;

	}
	else
	{
		private extern (C) int wcslen(const wchar* s);


		private alias WIN32_FIND_DATAW FIND_DATA;

	}
}
version (Posix)
{
	private import tango.stdc.stdio;

	private import tango.stdc.string;

	private import tango.stdc.posix.utime;

	private import tango.stdc.posix.dirent;

}
package struct FS
{
	struct Stamps
	{
		Time created;
		Time accessed;
		Time modified;
	}
	struct Listing
	{
		const(char)[] folder;
		bool allFiles;
		int opApply(scope int delegate(ref FileInfo) dg);
	}
	static void exception(const(char)[] filename);

	static void exception(const(char)[] prefix, const(char)[] error);

	static inout(char)[] padded(inout(char)[] path, char c = '/');

	static inout(char)[] paddedLeading(inout(char)[] path, char c = '/');

	static inout(char)[] stripped(inout(char)[] path, char c = '/');

	static char[] join(const(char[])[] paths...);

	static char[] strz(const(char)[] src, char[] dst);

	version (Win32)
	{
		private static wchar[] toString16(wchar[] tmp, const(char)[] path);


		private static char[] toString(char[] tmp, const(wchar[]) path);


		private static bool fileInfo(const(char)[] name, ref WIN32_FILE_ATTRIBUTE_DATA info);


		private static DWORD getInfo(const(char)[] name, ref WIN32_FILE_ATTRIBUTE_DATA info);


		private static DWORD getFlags(const(char)[] name);


		static bool exists(const(char)[] name);

		static ulong fileSize(const(char)[] name);

		static bool isWritable(const(char)[] name);

		static bool isFolder(const(char)[] name);

		static bool isFile(const(char)[] name);

		static Stamps timeStamps(const(char)[] name);

		static void timeStamps(const(char)[] name, Time accessed, Time modified);

		static void copy(const(char)[] src, const(char)[] dst);

		static bool remove(const(char)[] name);

		static void rename(const(char)[] src, const(char)[] dst);

		static void createFile(const(char)[] name);

		static void createFolder(const(char)[] name);

		static int list(const(char)[] folder, scope int delegate(ref FileInfo) dg, bool all = false);

		private static void createFile(const(char)[] name, scope void delegate(HANDLE) dg);


	}
	version (Posix)
	{
		private static uint getInfo(const(char)[] name, ref stat_t stats);


		static bool exists(const(char)[] name);

		static ulong fileSize(const(char)[] name);

		static bool isWritable(const(char)[] name);

		static bool isFolder(const(char)[] name);

		static bool isFile(const(char)[] name);

		static Stamps timeStamps(const(char)[] name);

		static void timeStamps(const(char)[] name, Time accessed, Time modified);

		static void copy(const(char)[] source, const(char)[] dest);

		static bool remove(const(char)[] name);

		static void rename(const(char)[] src, const(char)[] dst);

		static void createFile(const(char)[] name);

		static void createFolder(const(char)[] name);

		static int list(const(char)[] folder, scope int delegate(ref FileInfo) dg, bool all = false);

	}
}

template PathParser(char_t = char)
{
	struct PathParser
	{
		package char_t[] fp;

		package 
		{
			int end_;
			int ext_;
			int name_;
			int folder_;
			int suffix_;
		}
		PathParser parse(char_t[] path)
		{
			return parse(path, path.length);
		}

		@property const PathParser dup()
		{
			PathParser ret;
			ret.fp = fp.dup;
			ret.end_ = end_;
			ret.ext_ = ext_;
			ret.name_ = name_;
			ret.folder_ = folder_;
			ret.suffix_ = suffix_;
			return ret;
		}


		inout inout(char_t)[] dString()
		{
			return fp[0..end_];
		}

		const immutable(char)[] toString()
		{
			return dString().idup;
		}

		@property inout inout(char_t)[] root()
		{
			return fp[0..folder_];
		}


		@property inout inout(char_t)[] folder()
		{
			return fp[folder_..name_];
		}


		@property inout inout(char_t)[] parent()
		{
			auto p = path;
			if (name.length is 0)
			{
				for (int i = cast(int)p.length - 1;
				 --i > 0;)
				{
					if (p[i] is FileConst.PathSeparatorChar)
					{
						p = p[0..i];
						break;
					}
				}
			}
			return FS.stripped(p);
		}


		inout inout(char_t)[] pop()
		{
			return FS.stripped(path);
		}

		@property inout inout(char_t)[] name()
		{
			return fp[name_..suffix_];
		}


		@property char_t[] ext()
		{
			auto x = suffix;
			if (x.length)
			{
				if (ext_ is 0)
					foreach (c; x)
					{
						if (c is '.')
							++ext_;
						else
							break;
					}
				x = x[ext_..$];
			}
			return x;
		}


		@property inout inout(char_t)[] suffix()
		{
			return fp[suffix_..end_];
		}


		@property inout inout(char_t)[] path()
		{
			return fp[0..name_];
		}


		@property inout inout(char_t)[] file()
		{
			return fp[name_..end_];
		}


		const @property bool isAbsolute()
		{
			return folder_ > 0 || folder_ < end_ && fp[folder_] is FileConst.PathSeparatorChar;
		}


		const @property bool isEmpty()
		{
			return end_ is 0;
		}


		const @property bool isChild()
		{
			return folder().length > 0;
		}


		const bool equals(const(char)[] s)
		{
			return FS.stripped(s) == FS.stripped(dString());
		}


		package PathParser parse(char_t[] path, size_t end)
		{
			end_ = cast(int)end;
			fp = path;
			folder_ = 0;
			name_ = (suffix_ = -1);
			{
				for (int i = end_;
				 --i >= 0;)
				{
					switch (fp[i])
					{
						case FileConst.FileSeparatorChar:
						{
							if (name_ < 0)
								if (suffix_ < 0 && i && fp[i - 1] != '.')
									suffix_ = i;
							break;
						}
						case FileConst.PathSeparatorChar:
						{
							if (name_ < 0)
								name_ = i + 1;
							break;
						}
						case '\x5c':
						{
							FS.exception("unexpected '\\' character in path: ", path[0..end]);
							break;
							version (Win32)
							{
								case ':':
								{
									folder_ = i + 1;
									break;
								}
							}

						}
						default:
						{
							break;
						}
					}
				}
			}
			if (name_ < 0)
				name_ = folder_;
			if (suffix_ < 0 || suffix_ is name_)
				suffix_ = end_;
			return this;
		}


	}
}
bool exists(const(char)[] name);
Time modified(const(char)[] name);
Time accessed(const(char)[] name);
Time created(const(char)[] name);
ulong fileSize(const(char)[] name);
bool isWritable(const(char)[] name);
bool isFolder(const(char)[] name);
bool isFile(const(char)[] name);
FS.Stamps timeStamps(const(char)[] name);
void timeStamps(const(char)[] name, Time accessed, Time modified);
bool remove(const(char)[] name);
char[][] remove(char[][] paths);
void createFile(const(char)[] name);
void createFolder(const(char)[] name);
void createPath(const(char)[] path);
void rename(const(char)[] src, const(char)[] dst);
void copy(const(char)[] src, const(char)[] dst);
FS.Listing children(const(char)[] path, bool all = false);
char[][] collate(const(char)[] path, const(char)[] pattern, bool recurse = false);
char[] join(const(char[])[] paths...);
char[] standard(char[] path);
char[] native(char[] path);
inout(char)[] parent(inout(char)[] path);
inout(char)[] pop(inout(char)[] path);
inout(char)[] split(inout(char)[] path, out inout(char)[] head, out inout(char)[] tail);
char[] replace(char[] path, char from, char to);
template parse(char_t)
{
	PathParser!(char_t) parse(char_t[] path)
	{
		PathParser!(char_t) p;
		p.parse(path);
		return p;
	}

}
debug (UnitTest)
{
}
bool patternMatch(const(char)[] filename, const(char)[] pattern);
debug (UnitTest)
{
}
char[] normalize(const(char)[] in_path, char[] buf = null);
debug (UnitTest)
{
}
debug (Path)
{
	import tango.io.Stdout;
	void main();
}
