// D import file generated from 'tango/io/vfs/ZipFolder.d'
module tango.io.vfs.ZipFolder;
import Path = tango.io.Path;
import tango.io.device.File : File;
import tango.io.FilePath : FilePath;
import tango.io.device.TempFile : TempFile;
import tango.util.compress.Zip : ZipReader, ZipBlockReader, ZipWriter, ZipBlockWriter, ZipEntry, ZipEntryInfo, Method;
import tango.io.model.IConduit : IConduit, InputStream, OutputStream;
import tango.io.vfs.model.Vfs : VfsFolder, VfsFolderEntry, VfsFile, VfsFolders, VfsFiles, VfsFilter, VfsStats, VfsFilterInfo, VfsInfo, VfsSync;
import tango.time.Time : Time;
debug (ZipFolder)
{
	import tango.io.Stdout : Stderr;
}
version = Bug_HeapCorruption;
private 
{
	enum EntryType 
	{
		Dir,
		File,
	}
	struct Entry
	{
		EntryType type;
		union
		{
			DirEntry dir;
			FileEntry file;
		}
		const(char)[] fullname;
		const(char)[] name;
		VfsFilterInfo vfsFilterInfo;
		VfsInfo vfsInfo();
		void makeVfsInfo();
		@property bool isDir();

		@property bool isFile();

		@property ulong fileSize();

		InputStream openInput();
		OutputStream openOutput();
		void dispose();
		void dispose_children();
	}
	struct DirEntry
	{
		Entry*[const(char)[]] children;
	}
	struct FileEntry
	{
		ZipEntry zipEntry;
		TempFile tempFile;
	}
}
class ZipSubFolder : VfsFolder, VfsSync
{
	final @property const(char)[] name();

	final override string toString();

	final @property VfsFile file(const(char)[] path);

	final @property VfsFolderEntry folder(const(char)[] path);

	final @property VfsFolders self();

	final @property VfsFolders tree();

	final int opApply(scope int delegate(ref VfsFolder) dg);

	final VfsFolder clear();

	final @property bool writable();

	VfsFolder close(bool commit = true);
	override VfsFolder sync();

	final void verify(VfsFolder folder, bool mounting);

	final @property ZipFolder archive();

	private 
	{
		ZipFolder _archive;
		Entry* entry;
		VfsStats stats;
		final @property ZipFolder archive(ZipFolder v);

		this(ZipFolder archive, Entry* entry);
		final void reset(ZipFolder archive, Entry* entry);

		final @property bool valid();

		final void enforce_mutable();

		final void mutate();

		final ZipSubFolder[] folders(bool collect);

		final Entry*[] files(ref VfsStats stats, VfsFilter filter = null);

	}
}
class ZipFolder : ZipSubFolder
{
	this(const(char)[] path, bool readonly = false);
	final override VfsFolder close(bool commit = true);

	final override VfsFolder sync();

	final @property bool readonly();

	final @property const(char)[] path();

	final @property const(char)[] path(const(char)[] v);

	private 
	{
		ZipReader zr;
		Entry* root;
		const(char)[] _path;
		bool _readonly;
		bool modified = false;
		final @property bool readonly(bool v);

		final @property bool closed();

		final @property bool valid();

		final OutputStream mutateStream(OutputStream source);

		void mutate_write(size_t bytes, const(void)[] src);
		void resetArchive(const(char)[] path, bool readonly = false);
	}
}
class ZipFile : VfsFile
{
	final @property const(char)[] name();

	final override string toString();

	final @property bool exists();

	final @property ulong size();

	final VfsFile copy(VfsFile source);

	final VfsFile move(VfsFile source);

	final VfsFile create();

	final VfsFile create(InputStream stream);

	final VfsFile remove();

	final @property InputStream input();

	final @property OutputStream output();

	final @property VfsFile dup();

	final @property Time modified();

	private 
	{
		ZipFolder archive;
		Entry* entry;
		Entry* parent;
		const(char)[] name_;
		this();
		this(ZipFolder archive, Entry* parent, Entry* entry);
		this(ZipFolder archive, Entry* parent, const(char)[] name);
		final @property bool valid();

		final void enforce_mutable();

		final void mutate();

		final void reset(ZipFolder archive, Entry* parent, Entry* entry);

		final void reset(ZipFolder archive, Entry* parent, const(char)[] name);

		final void close();

	}
}
class ZipSubFolderEntry : VfsFolderEntry
{
	final VfsFolder open();

	final VfsFolder create();

	final @property bool exists();

	private 
	{
		ZipFolder archive;
		Entry* parent;
		const(char)[] name;
		this(ZipFolder archive, Entry* parent, const(char)[] name);
		final @property bool valid();

		final void enforce_mutable();

		final void mutate();

	}
}
class ZipSubFolderGroup : VfsFolders
{
	final int opApply(scope int delegate(ref VfsFolder) dg);

	final @property size_t files();

	final @property size_t folders();

	final @property size_t entries();

	final @property ulong bytes();

	final VfsFolders subset(const(char)[] pattern);

	final @property VfsFiles catalog(const(char)[] pattern);

	final @property VfsFiles catalog(VfsFilter filter = null);

	private 
	{
		ZipFolder archive;
		ZipSubFolder[] members;
		this(ZipFolder archive, ZipSubFolder root, bool recurse);
		this(ZipFolder archive, ZipSubFolder[] members);
		final @property bool valid();

		final ZipSubFolder[] scan(ZipSubFolder root, bool recurse);

	}
}
class ZipFileGroup : VfsFiles
{
	final int opApply(scope int delegate(ref VfsFile) dg);

	final @property size_t files();

	final @property ulong bytes();

	private 
	{
		ZipFolder archive;
		FileEntry[] group;
		VfsStats stats;
		struct FileEntry
		{
			Entry* parent;
			Entry* entry;
		}
		this(ZipFolder archive, ZipSubFolderGroup host, VfsFilter filter);
		final @property bool valid();

	}
}
private 
{
	void error(const(char)[] msg);
	void mutate_error(const(char)[] method);
	bool nz(const(char)[] s);
	bool zero(const(char)[] s);
	bool single_path_part(const(char)[] s);
	char[] dir_app(const(char)[] dir, const(char)[] name);
	inout(char)[] headTail(inout(char)[] path, out inout(char)[] head, out inout(char)[] tail);
	debug (UnitTest)
	{
	}
	private 
	{
		import tango.io.device.Conduit : Conduit;
		private deprecated class DummyInputStream : InputStream
		{
			override InputStream input();

			override IConduit conduit();

			override void close();

			override size_t read(void[] dst);

			override InputStream flush();

			override void[] load(size_t max = -1);

			override long seek(long offset, Anchor anchor = cast(Anchor)0);

		}


		private deprecated class DummyOutputStream : OutputStream
		{
			override OutputStream output();

			override IConduit conduit();

			override void close();

			override size_t write(const(void)[] src);

			override OutputStream copy(InputStream src, size_t max = -1);

			override OutputStream flush();

			override long seek(long offset, Anchor anchor = cast(Anchor)0);

		}


		class EventSeekInputStream : InputStream
		{
			struct Callbacks
			{
				void delegate() close;
				void delegate() clear;
				void delegate(size_t, void[]) read;
				void delegate(long, long, Anchor) seek;
			}
			this(InputStream source, Callbacks callbacks);
			override IConduit conduit();

			InputStream input();
			override void close();

			override size_t read(void[] dst);

			override InputStream flush();

			override void[] load(size_t max = -1);

			override long seek(long offset, Anchor anchor = cast(Anchor)0);

			private 
			{
				InputStream source;
				InputStream seeker;
				Callbacks callbacks;
			}
		}
		class EventSeekOutputStream : OutputStream
		{
			struct Callbacks
			{
				void delegate() close;
				void delegate() flush;
				void delegate(size_t, const(void)[]) write;
				void delegate(long, long, Anchor) seek;
			}
			this(OutputStream source, Callbacks callbacks);
			override IConduit conduit();

			override OutputStream output();

			override void close();

			override size_t write(const(void)[] dst);

			override OutputStream flush();

			override long seek(long offset, Anchor anchor = cast(Anchor)0);

			override OutputStream copy(InputStream src, size_t max = -1);

			private 
			{
				OutputStream source;
				OutputStream seeker;
				Callbacks callbacks;
			}
		}
		class WrapSeekInputStream : InputStream
		{
			this(InputStream source);
			this(InputStream source, long position);
			override IConduit conduit();

			InputStream input();
			override void close();

			override size_t read(void[] dst);

			override InputStream flush();

			override void[] load(size_t max = -1);

			override long seek(long offset, Anchor anchor = cast(Anchor)0);

			private 
			{
				InputStream source;
				InputStream seeker;
				long _position;
			}
		}
		class WrapSeekOutputStream : OutputStream
		{
			this(OutputStream source);
			this(OutputStream source, long position);
			override IConduit conduit();

			override OutputStream output();

			override void close();

			size_t write(const(void)[] src);
			override OutputStream copy(InputStream src, size_t max = -1);

			override OutputStream flush();

			override long seek(long offset, Anchor anchor = cast(Anchor)0);

			private 
			{
				OutputStream source;
				OutputStream seeker;
				long _position;
			}
		}
	}
}
