// D import file generated from 'tango/net/ftp/FtpClient.d'
module tango.net.ftp.FtpClient;
private 
{
	import tango.net.ftp.Telnet;
	import tango.net.device.Berkeley;
	import tango.text.Util;
	import tango.time.Clock;
	import tango.text.Regex : Regex;
	import tango.time.chrono.Gregorian;
	import tango.core.Array;
	import tango.net.device.Socket;
	import tango.io.device.Conduit;
	import tango.io.device.Array;
	import tango.io.device.File;
	import Text = tango.text.Util;
	import Ascii = tango.text.Ascii;
	import Integer = tango.text.convert.Integer;
	import Timestamp = tango.text.convert.TimeStamp;
}
alias void delegate(in size_t pos) FtpProgress;
enum FtpFormat 
{
	ascii,
	image,
}
struct FtpAddress
{
	static FtpAddress* opCall(const(char)[] str);

	const(char)[] address;
	const(char)[] directory;
	const(char)[] user = "anonymous";
	const(char)[] pass = "anonymous@anonymous";
	uint port = 21;
}
struct FtpResponse
{
	char[3] code = "000";
	const(char)[] message = null;
}
enum FtpConnectionType 
{
	active,
	passive,
}
struct FtpConnectionDetail
{
	FtpConnectionType type = FtpConnectionType.passive;
	Address address = null;
	Address listen = null;
}
struct FtpFeature
{
	const(char)[] command = null;
	const(char)[] params = null;
}
enum FtpFileType 
{
	unknown,
	file,
	cdir,
	pdir,
	dir,
	other,
}
struct FtpFileInfo
{
	const(char)[] name = null;
	FtpFileType type = FtpFileType.unknown;
	ulong size = (ulong).max;
	Time modify = Time.max;
	Time create = Time.max;
	const(char)[] mime = null;
	const(char)[][const(char)[]] facts;
}
class FtpException : Exception
{
	char[3] responseCode_ = "000";
	this(string message, char[3] code = "420");
	this(FtpResponse r);
	override string toString();

}
class FTPConnection : Telnet
{
	FtpFeature[] supportedFeatures_ = null;
	FtpConnectionDetail inf_;
	size_t restartPos_ = 0;
	const(char)[] currFile_ = "";
	Socket dataSocket_;
	TimeSpan timeout_ = TimeSpan.fromMillis(5000);
	@property public TimeSpan timeout();


	@property public void timeout(TimeSpan t);


	public TimeSpan shutdownTime();

	public FtpFeature[] supportedFeatures();

	override void exception(string message);

	void exception(FtpResponse fr);
	public this();

	public this(const(char)[] hostname, const(char)[] username = "anonymous", const(char)[] password = "anonymous@anonymous", uint port = 21);

	public this(FtpAddress fad);

	public void connect(FtpAddress fad);

	public void connect(const(char)[] hostname, const(char)[] username = "anonymous", const(char)[] password = "anonymous@anonymous", uint port = 21);

	public void close();

	public void setPassive();

	public void setActive(const(char)[] ip, ushort port, const(char)[] listen_ip = null, ushort listen_port = 0);

	public void cd(const(char)[] dir);

	public void cdup();

	public const(char)[] cwd();

	public void chmod(const(char)[] path, int mode);

	public void del(const(char)[] path);

	public void rm(const(char)[] path);

	public void rename(const(char)[] old_path, const(char)[] new_path);

	int exist(const(char)[] file);
	public size_t size(const(char)[] path, FtpFormat format = FtpFormat.image);

	public void type(FtpFormat format);

	Time modified(const(char)[] file);
	protected Time parseTimeval(const(char)[] timeval);

	public void noop();

	public const(char)[] mkdir(const(char)[] path);

	public void getFeatures();

	public void sendCommand(const(char)[] command, const(char)[][] parameters...);

	public FtpResponse readResponse(const(char)[] expected_code);

	public FtpResponse readResponse();

	protected const(char)[] parse257(FtpResponse response);

	protected Socket getDataSocket();

	version (TangoDoc)
	{
		public Socket connectPassive();

	}
	else
	{
		public Socket connectPassive();

	}
	public bool isSupported(const(char)[] command);

	public bool is_supported(const(char)[] command);

	protected void prepareDataSocket(ref Socket data);

	public void finishDataCommand(Socket data);

	public Socket processDataCommand(const(char)[] command, const(char)[][] parameters...);

	public FtpFileInfo[] ls(const(char)[] path = "");

	protected void readStream(Socket data, OutputStream stream, FtpProgress progress = null);

	protected void sendStream(Socket data, InputStream stream, FtpProgress progress = null);

	protected FtpFileInfo[] sendListCommand(const(char)[] path);

	protected FtpFileInfo parseListLine(const(char)[] line);

	protected FtpFileInfo parseMlstLine(const(char)[] line);

	public FtpFileInfo getFileInfo(const(char)[] path);

	public void put(const(char)[] path, const(char)[] local_file, FtpProgress progress = null, FtpFormat format = FtpFormat.image);

	public void put(const(char)[] path, InputStream stream = null, FtpProgress progress = null, FtpFormat format = FtpFormat.image);

	public void append(const(char)[] path, InputStream stream, FtpProgress progress = null, FtpFormat format = FtpFormat.image);

	public void restartSeek(size_t offset);

	public void allocate(long bytes);

	public void get(const(char)[] path, const(char)[] local_file, FtpProgress progress = null, FtpFormat format = FtpFormat.image);

	public void enableUTF8();

	public void get(const(char)[] path, OutputStream stream, FtpProgress progress = null, FtpFormat format = FtpFormat.image);

	public InputStream input(const(char)[] path);

	public OutputStream output(const(char)[] path);

}
