// D import file generated from 'tango/sys/SharedLib.d'
module tango.sys.SharedLib;
private 
{
	import tango.stdc.stringz : fromStringz;
	version (Windows)
	{
		import tango.sys.Common : SysError;
		import tango.sys.win32.Types : HINSTANCE, HMODULE, BOOL;
		extern (Windows) 
		{
			void* GetProcAddress(HINSTANCE, const(char)*);
			BOOL FreeLibrary(HMODULE);
			version (Win32SansUnicode)
			{
				HINSTANCE LoadLibraryA(const(char)*);
			}
			else
			{
				enum 
				{
					CP_UTF8 = 65001,
				}
				HINSTANCE LoadLibraryW(const(wchar)*);
				int MultiByteToWideChar(uint, uint, const(char)*, int, wchar*, int);
			}
		}
	}
	else
	{
		version (Posix)
		{
			import tango.stdc.posix.dlfcn;
		}
		else
		{
			static assert(false, "No support for this platform");
		}
	}
	version (SharedLibVerbose)
	{
		import tango.util.log.Trace;
	}
}
version (Posix)
{
	version (FreeBSD)
	{
	}
	else
	{
		pragma (lib, "dl");
	}
}
final class SharedLib
{
	enum LoadMode 
	{
		Now = 1,
		Lazy = 2,
		Global = 4,
		Local = 8,
	}
	static SharedLib load(const(char)[] path, LoadMode mode = LoadMode.Now | LoadMode.Global);

	static SharedLib loadNoThrow(const(char)[] path, LoadMode mode = LoadMode.Now | LoadMode.Global);

	private static SharedLib loadImpl(const(char)[] path, LoadMode mode, bool throwExceptions);


	void unload();
	void unloadNoThrow();
	private void unloadImpl(bool throwExceptions);

	@property const(char)[] path();

	void* getSymbol(const(char)* name);
	void* getSymbolNoThrow(const(char)* name);
	private void* getSymbolImpl(const(char)* name, bool throwExceptions);

	static uint numLoadedLibs();

	private 
	{
		version (Windows)
		{
			HMODULE handle;
			void load_(LoadMode mode, bool throwExceptions);
			void* getSymbol_(const(char)* name, bool throwExceptions);
			void unload_(bool throwExceptions);
		}
		else
		{
			version (Posix)
			{
				void* handle;
				void load_(LoadMode mode, bool throwExceptions);
				void* getSymbol_(const(char)* name, bool throwExceptions);
				void unload_(bool throwExceptions);
			}
			else
			{
				static assert(false, "No support for this platform");
			}
		}
		const(char)[] path_;
		int refCnt = 0;
		@property bool loaded();

		this(const(char)[] path);
	}
	private __gshared 
	{
		SharedLib[char[]] loadedLibs;
		Object mutex;
	}

	shared static this();
}

class SharedLibException : Exception
{
	this(immutable(char)[] msg);
}
debug (SharedLib)
{
	void main();
}
