// D import file generated from 'tango/time/Time.d'
module tango.time.Time;
struct TimeSpan
{
	package long ticks_;

	enum : long
	{
		NanosecondsPerTick = 100,
		TicksPerMicrosecond = 1000 / NanosecondsPerTick,
		TicksPerMillisecond = 1000 * TicksPerMicrosecond,
		TicksPerSecond = 1000 * TicksPerMillisecond,
		TicksPerMinute = 60 * TicksPerSecond,
		TicksPerHour = 60 * TicksPerMinute,
		TicksPerDay = 24 * TicksPerHour,
		MillisPerSecond = 1000,
		MillisPerMinute = MillisPerSecond * 60,
		MillisPerHour = MillisPerMinute * 60,
		MillisPerDay = MillisPerHour * 24,
		DaysPerYear = 365,
		DaysPer4Years = DaysPerYear * 4 + 1,
		DaysPer100Years = DaysPer4Years * 25 - 1,
		DaysPer400Years = DaysPer100Years * 4 + 1,
		Epoch1601 = DaysPer400Years * 4 * TicksPerDay,
		Epoch1970 = Epoch1601 + TicksPerSecond * 11644473600L,
	}
	enum TimeSpan min = TimeSpan((long).min);
	enum TimeSpan max = TimeSpan((long).max);
	enum TimeSpan zero = TimeSpan(0);
	const @property long ticks();

	const bool opEquals(const(TimeSpan) t);

	const int opCmp(ref const TimeSpan t);
	const int opCmp(const TimeSpan t);
	const template opBinary(immutable(char)[] o : "+")
	{
		TimeSpan opBinary(const(TimeSpan) t)
		{
			return TimeSpan(ticks_ + t.ticks_);
		}

	}

	template opOpAssign(immutable(char)[] o : "+")
	{
		TimeSpan opOpAssign(const(TimeSpan) t)
		{
			ticks_ += t.ticks_;
			return this;
		}

	}
	const template opBinary(immutable(char)[] o : "-")
	{
		TimeSpan opBinary(const(TimeSpan) t)
		{
			return TimeSpan(ticks_ - t.ticks_);
		}

	}

	template opOpAssign(immutable(char)[] o : "-")
	{
		TimeSpan opOpAssign(const(TimeSpan) t)
		{
			ticks_ -= t.ticks_;
			return this;
		}

	}
	const template opBinary(immutable(char)[] o : "*")
	{
		TimeSpan opBinary(long v)
		{
			return TimeSpan(ticks_ * v);
		}

	}

	template opOpAssign(immutable(char)[] o : "*")
	{
		TimeSpan opOpAssign(long v)
		{
			ticks_ *= v;
			return this;
		}

	}
	const template opBinary(immutable(char)[] o : "/")
	{
		TimeSpan opBinary(long v)
		{
			return TimeSpan(ticks_ / v);
		}

	}

	template opOpAssign(immutable(char)[] o : "/")
	{
		TimeSpan opOpAssign(long v)
		{
			ticks_ /= v;
			return this;
		}

	}
	const template opBinary(immutable(char)[] o : "/")
	{
		long opBinary(const(TimeSpan) t)
		{
			return ticks_ / t.ticks;
		}

	}

	const template opUnary(immutable(char)[] o : "-")
	{
		TimeSpan opUnary()
		{
			return TimeSpan(-ticks_);
		}

	}

	const @property long nanos();

	const @property long micros();

	const @property long millis();

	const @property long seconds();

	const @property long minutes();

	const @property long hours();

	const @property long days();

	const @property double interval();

	const @property TimeOfDay time();

	static TimeSpan fromNanos(long value);

	static TimeSpan fromMicros(long value);

	static TimeSpan fromMillis(long value);

	static TimeSpan fromSeconds(long value);

	static TimeSpan fromMinutes(long value);

	static TimeSpan fromHours(long value);

	static TimeSpan fromDays(long value);

	static TimeSpan fromInterval(double sec);

}
struct Time
{
	private long ticks_;

	private enum : long
	{
		maximum = (TimeSpan.DaysPer400Years * 25 - 366) * TimeSpan.TicksPerDay - 1,
		minimum = -((TimeSpan.DaysPer400Years * 25 - 366) * TimeSpan.TicksPerDay - 1),
	}

	enum Time min = Time(minimum);
	enum Time max = Time(maximum);
	enum Time epoch = Time(0L);
	enum Time epoch1601 = Time(TimeSpan.Epoch1601);
	enum Time epoch1970 = Time(TimeSpan.Epoch1970);
	const @property long ticks();

	const bool opEquals(const(Time) t);

	const int opCmp(ref const Time t);
	const int opCmp(const Time t);
	const template opBinary(immutable(char)[] o : "+")
	{
		Time opBinary(const(TimeSpan) t)
		{
			return Time(ticks_ + t.ticks_);
		}

	}

	template opOpAssign(immutable(char)[] o : "+")
	{
		Time opOpAssign(const(TimeSpan) t)
		{
			ticks_ += t.ticks_;
			return this;
		}

	}
	const template opBinary(immutable(char)[] o : "-")
	{
		Time opBinary(const(TimeSpan) t)
		{
			return Time(ticks_ - t.ticks_);
		}

	}

	const template opBinary(immutable(char)[] o : "-")
	{
		TimeSpan opBinary(const(Time) t)
		{
			return TimeSpan(ticks_ - t.ticks_);
		}

	}

	template opOpAssign(immutable(char)[] o : "-")
	{
		Time opOpAssign(const(TimeSpan) t)
		{
			ticks_ -= t.ticks_;
			return this;
		}

	}
	const @property Time date();

	const @property TimeOfDay time();

	const @property TimeSpan span();

	const @property TimeSpan unix();

}
struct TimeOfDay
{
	public uint hours;

	public uint minutes;

	public uint seconds;

	public uint millis;

	static TimeOfDay opCall(uint hours, uint minutes, uint seconds, uint millis = 0);

	static TimeOfDay opCall(long ticks);

	const @property TimeSpan span();

	package static TimeSpan modulo24(long ticks);


}
struct Date
{
	public 
	{
		uint era;
		uint day;
		uint year;
		uint month;
		uint dow;
		uint doy;
	}
}
struct DateTime
{
	public Date date;

	public TimeOfDay time;

}
debug (UnitTest)
{
}
debug (Time)
{
	import tango.io.Stdout;
	import tango.time.Clock;
	import tango.time.chrono.Gregorian;
	Time foo();
	void main();
}
