// D import file generated from 'tango/util/compress/Zip.d'
module tango.util.compress.Zip;
import tango.core.ByteSwap : ByteSwap;
import tango.io.device.Array : Array;
import tango.io.device.File : File;
import Path = tango.io.Path;
import tango.io.device.FileMap : FileMap;
import tango.io.stream.Zlib : ZlibInput, ZlibOutput;
import tango.util.digest.Crc32 : Crc32;
import tango.io.model.IConduit : IConduit, InputStream, OutputStream;
import tango.io.stream.Digester : DigestInput;
import tango.time.Time : Time, TimeSpan;
import tango.time.WallClock : WallClock;
import tango.time.chrono.Gregorian : Gregorian;
import Integer = tango.text.convert.Integer;
debug (Zip)
{
	import tango.io.Stdout : Stderr;
}
private 
{
	align (1)struct LocalFileHeaderData
	{
		align (1)
		{
			ushort extract_version = (ushort).max;
			ushort general_flags = 0;
			ushort compression_method = 0;
			ushort modification_file_time = 0;
			ushort modification_file_date = 0;
			uint crc_32 = 0;
			uint compressed_size = 0;
			uint uncompressed_size = 0;
			ushort file_name_length = 0;
			ushort extra_field_length = 0;
			debug (Zip)
			{
				void dump();
			}
		}
	}

	struct LocalFileHeader
	{
		enum uint signature = 67324752;
		alias LocalFileHeaderData Data;
		Data data;
		static assert(Data.sizeof == 26);
		char[] file_name;
		ubyte[] extra_field;
		void[] data_arr();
		void put(OutputStream output);
		void fill(InputStream src);
		bool agrees_with(FileHeader h);
	}
	align (1)struct FileHeaderData
	{
		align (1)
		{
			ubyte zip_version;
			ubyte file_attribute_type;
			ushort extract_version;
			ushort general_flags;
			ushort compression_method;
			ushort modification_file_time;
			ushort modification_file_date;
			uint crc_32;
			uint compressed_size;
			uint uncompressed_size;
			ushort file_name_length;
			ushort extra_field_length;
			ushort file_comment_length;
			ushort disk_number_start;
			ushort internal_file_attributes = 0;
			uint external_file_attributes = 0;
			int relative_offset_of_local_header;
			debug (Zip)
			{
				void dump();
			}
			void fromLocal(LocalFileHeader.Data data);
		}
	}

	struct FileHeader
	{
		enum uint signature = 33639248;
		alias FileHeaderData Data;
		Data* data;
		static assert(Data.sizeof == 42);
		const(char)[] file_name;
		ubyte[] extra_field;
		const(char)[] file_comment;
		bool usingDataDescriptor();
		uint compressionOptions();
		bool usingUtf8();
		void[] data_arr();
		void put(OutputStream output);
		long map(void[] src);
	}
	align (1)struct EndOfCDRecordData
	{
		align (1)
		{
			ushort disk_number = 0;
			ushort disk_with_start_of_central_directory = 0;
			ushort central_directory_entries_on_this_disk;
			ushort central_directory_entries_total;
			uint size_of_central_directory;
			uint offset_of_start_of_cd_from_starting_disk;
			ushort file_comment_length;
			debug (Zip)
			{
				void dump();
			}
		}
	}

	struct EndOfCDRecord
	{
		enum uint signature = 101010256;
		alias EndOfCDRecordData Data;
		Data data;
		static assert(data.sizeof == 18);
		char[] file_comment;
		void[] data_arr();
		void put(OutputStream output);
		void fill(void[] src);
	}
}
public enum Method 
{
	Store,
	Deflate,
	Unsupported,
}

private 
{
	const ushort ZIP_VERSION = 20;

	const ushort MAX_EXTRACT_VERSION = 20;

	const ushort SUPPORTED_FLAGS = 14;

	const ushort UNSUPPORTED_FLAGS = ~SUPPORTED_FLAGS;

	Method toMethod(ushort method);
	ushort fromMethod(Method method);
}
interface ZipReader
{
	bool streamed();
	void close();
	bool more();
	ZipEntry get();
	ZipEntry get(ZipEntry);
	int opApply(int delegate(ref ZipEntry));
}
interface ZipWriter
{
	void finish();
	void putFile(ZipEntryInfo info, const(char)[] path);
	void putStream(ZipEntryInfo info, InputStream source);
	void putEntry(ZipEntryInfo info, ZipEntry entry);
	void putData(ZipEntryInfo info, const(void)[] data);
	Method method();
	Method method(Method);
}
class ZipBlockReader : ZipReader
{
	this(const(char)[] path);
	this(InputStream source);
	bool streamed();
	void close();
	bool more();
	ZipEntry get();
	ZipEntry get(ZipEntry reuse);
	int opApply(int delegate(ref ZipEntry) dg);
	private 
	{
		InputStream source;
		InputStream seeker;
		enum State 
		{
			Init,
			Open,
			Done,
		}
		State state;
		size_t current_index = 0;
		FileHeader[] headers;
		ubyte[] cd_data;
		File file_source = null;
		void read_cd();
		EndOfCDRecord read_eocd_record();
		InputStream open_file(FileHeader header, bool raw);
		InputStream open_file_raw(FileHeader header);
	}
}
class ZipBlockWriter : ZipWriter
{
	this(const(char)[] path);
	this(OutputStream output);
	void finish();
	void putFile(ZipEntryInfo info, const(char)[] path);
	void putStream(ZipEntryInfo info, InputStream source);
	void putEntry(ZipEntryInfo info, ZipEntry entry);
	void putData(ZipEntryInfo info, const(void)[] data);
	@property Method method();

	@property Method method(Method v);

	private 
	{
		OutputStream output;
		OutputStream seeker;
		File file_output;
		Method _method;
		struct Entry
		{
			FileHeaderData data;
			long header_position;
			const(char)[] filename;
			const(char)[] comment;
			ubyte[] extra;
		}
		Entry[] entries;
		void put_cd();
		void put_raw(ZipEntryInfo info, ZipEntry entry);
		void put_compressed(ZipEntryInfo info, InputStream source);
		void patch_local_header(uint crc_32, uint compressed_size, uint uncompressed_size);
		void put_local_header(ZipEntryInfo info, Method method);
		void put_local_header(LocalFileHeaderData data, const(char)[] file_name);
	}
}
class ZipEntry
{
	ZipEntryInfo info;
	uint size();
	InputStream open();
	void verify();
	ZipEntry dup();
	private 
	{
		alias InputStream delegate(FileHeader, bool raw) open_dg_t;
		open_dg_t open_dg;
		FileHeader header;
		bool verified = false;
		InputStream open_raw();
		this(FileHeader header, open_dg_t open_dg);
		ZipEntry reset(FileHeader header, open_dg_t open_dg);
	}
}
struct ZipEntryInfo
{
	const(char)[] name;
	Time modified = Time.min;
	const(char)[] comment;
}
class ZipException : Exception
{
	this(immutable(char)[] msg);
	private 
	{
		alias typeof(this) thisT;
		static void opCall(immutable(char)[] msg);

		@property static void badsig();


		@property static void badsig(const(char)[] type);


		@property static void incons(const(char)[] name);


		@property static void missingdir();


		@property static void toomanyentries();


		@property static void toolong();


		@property static void cdtoolong();


		@property static void fntoolong();


		@property static void eftoolong();


		@property static void cotoolong();


		@property static void fnencode();


		@property static void coencode();


		@property static void tooold();


	}
}
class ZipChecksumException : ZipException
{
	this(const(char)[] name);
	private static void opCall(const(char)[] name);


}
class ZipExhaustedException : ZipException
{
	this();
	private static void opCall();


}
class ZipNotSupportedException : ZipException
{
	this(immutable(char)[] msg);
	private 
	{
		alias ZipNotSupportedException thisT;
		static void opCall(const(char)[] msg);

		static void spanned();

		static void zipver(ushort ver);

		static void flags();

		static void method(ushort m);

	}
}
void createArchive(const(char)[] archive, Method method, const(char[])[] files...);
void extractArchive(const(char)[] archive, const(char)[] dest);
private 
{
	class ZipEntryVerifier : InputStream
	{
		this(ZipEntry entry, InputStream source);
		IConduit conduit();
		InputStream input();
		long seek(long ofs, Anchor anchor = Anchor.Begin);
		void close();
		size_t read(void[] dst);
		override void[] load(size_t max = -1);

		override InputStream flush();

		private 
		{
			Crc32 digest;
			InputStream source;
			ZipEntry entry;
			void check();
		}
	}
	void readExact(InputStream s, void[] dst);
	void writeExact(OutputStream s, const(void)[] src);
	template write(T)
	{
		void write(OutputStream s, T value)
		{
			version (BigEndian)
			{
				swap(value);
			}

			writeExact(s, (&value)[0..1]);
		}

	}
	template swapAll(T)
	{
		void swapAll(ref T data)
		{
			static if (is(typeof(T.record_fields)))
			{
				const fields = T.record_fields;
			}
			else
			{
				const fields = data.tupleof.length;
			}

			foreach (i, _; data.tupleof)
			{
				if (i == fields)
					break;
				swap(data.tupleof[i]);
			}
		}

	}
	template swap(T)
	{
		void swap(ref T data)
		{
			static if (T.sizeof == 1)
			{
			}
			else
			{
				static if (T.sizeof == 2)
				{
					ByteSwap.swap16(&data, 2);
				}
				else
				{
					static if (T.sizeof == 4)
					{
						ByteSwap.swap32(&data, 4);
					}
					else
					{
						static if (T.sizeof == 8)
						{
							ByteSwap.swap64(&data, 8);
						}
						else
						{
							static if (T.sizeof == 10)
							{
								ByteSwap.swap80(&data, 10);
							}
							else
							{
								static assert(false, "Can't swap " ~ T.stringof ~ "s.");
							}

						}

					}

				}

			}

		}

	}
	const char[][] cp437_to_utf8_map_low = ["\x00"[], "\xe2\x98\xba", "\xe2\x98\xbb", "\xe2\x99\xa5", "\xe2\x99\xa6", "\xe2\x99\xa3", "\xe2\x99\xa0", "\xe2\x80\xa2", "\xe2\x97\x98", "\xe2\x97\x8b", "\xe2\x97\x99", "\xe2\x99\x82", "\xe2\x99\x80", "\xe2\x99\xaa", "\xe2\x99\xab", "\xe2\x98\xbc", "\xe2\x96\xb6", "\xe2\x97\x80", "\xe2\x86\x95", "\xe2\x80\xbc", "\xc2\xb6", "\xc2\xa7", "\xe2\x96\xac", "\xe2\x86\xa8", "\xe2\x86\x91", "\xe2\x86\x93", "\xe2\x86\x92", "\xe2\x86\x90", "\xe2\x88\x9f", "\xe2\x86\x94", "\xe2\x96\xb2", "\xe2\x96\xbc"];

	const char[][] cp437_to_utf8_map_high = ["\xc3\x87"[], "\xc3\xbc", "\xc3\xa9", "\xc3\xa2", "\xc3\xa4", "\xc3\xa0", "\xc3\xa5", "\xc3\xa7", "\xc3\xaa", "\xc3\xab", "\xc3\xa8", "\xc3\xaf", "\xc3\xae", "\xc3\xac", "\xc3\x84", "\xc3\x85", "\xc3\x89", "\xc3\xa6", "\xc3\x86", "\xc3\xb4", "\xc3\xb6", "\xc3\xb2", "\xc3\xbb", "\xc3\xb9", "\xc3\xbf", "\xc3\x96", "\xc3\x9c", "\xc3\xb8", "\xc2\xa3", "\xc2\xa5", "\xe2\x82\xa7", "\xc6\x92", "\xc3\xa1", "\xc3\xad", "\xc3\xb3", "\xc3\xba", "\xc3\xb1", "\xc3\x91", "\xc2\xaa", "\xc2\xba", "\xc2\xbf", "\xe2\x8c\x90", "\xc2\xac", "\xc2\xbd", "\xc2\xbc", "\xc2\xa1", "\xc2\xab", "\xc2\xbb", "\xe2\x96\x91", "\xe2\x96\x92", "\xe2\x96\x93", "\xe2\x94\x82", "\xe2\x94\xa4", "\xe2\x95\xa1", "\xe2\x95\xa2", "\xe2\x95\x96", "\xe2\x95\x95", "\xe2\x95\xa3", "\xe2\x95\x91", "\xe2\x95\x97", "\xe2\x95\x9d", "\xe2\x95\x9c", "\xe2\x95\x9b", "\xe2\x94\x90", "\xe2\x94\x94", "\xe2\x94\xb4", "\xe2\x94\xac", "\xe2\x94\x9c", "\xe2\x94\x80", "\xe2\x94\xbc", "\xe2\x95\x9e", "\xe2\x95\x9f", "\xe2\x95\x9a", "\xe2\x95\x94", "\xe2\x95\xa9", "\xe2\x95\xa6", "\xe2\x95\xa0", "\xe2\x95\x90", "\xe2\x95\xac", "\xe2\x95\xa7", "\xe2\x95\xa8", "\xe2\x95\xa4", "\xe2\x95\xa5", "\xe2\x95\x99", "\xe2\x95\x98", "\xe2\x95\x92", "\xe2\x95\x93", "\xe2\x95\xab", "\xe2\x95\xaa", "\xe2\x94\x98", "\xe2\x94\x8c", "\xe2\x96\x88", "\xe2\x96\x84", "\xe2\x96\x8c", "\xe2\x96\x90", "\xe2\x96\x80", "\xce\xb1", "\xc3\x9f", "\xce\x93", "\xcf\x80", "\xce\xa3", "\xcf\x83", "\xc2\xb5", "\xcf\x84", "\xce\xa6", "\xce\x98", "\xce\xa9", "\xce\xb4", "\xe2\x88\x9e", "\xcf\x86", "\xce\xb5", "\xe2\x88\xa9", "\xe2\x89\xa1", "\xc2\xb1", "\xe2\x89\xa5", "\xe2\x89\xa4", "\xe2\x8c\xa0", "\xe2\x8c\xa1", "\xc3\xb7", "\xe2\x89\x88", "\xc2\xb0", "\xe2\x88\x99", "\xc2\xb7", "\xe2\x88\x9a", "\xe2\x81\xbf", "\xc2\xb2", "\xe2\x96\xa0", "\xc2\xa0"];

	inout(char[]) cp437_to_utf8(inout(ubyte[]) s);
	debug (UnitTest)
	{
	}
	const __gshared char[dchar] utf8_to_cp437_map;

	shared static this();
	inout(ubyte)[] utf8_to_cp437(inout(char)[] s);
	debug (UnitTest)
	{
	}
	inout(char[]) utf8_to_utf8(inout(ubyte[]) s);
	ubyte[] utf8_to_utf8(char[] s);
	void dosToTime(ushort dostime, ushort dosdate, out Time time);
	void timeToDos(Time time, out ushort dostime, out ushort dosdate);
	private 
	{
		import tango.io.device.Conduit : Conduit;
		class CounterInput : InputStream
		{
			this(InputStream input);
			override IConduit conduit();

			InputStream input();
			long seek(long ofs, Anchor anchor = Anchor.Begin);
			override void close();

			override size_t read(void[] dst);

			override void[] load(size_t max = -1);

			override InputStream flush();

			long count();
			private 
			{
				InputStream source;
				long _count;
			}
		}
		class CounterOutput : OutputStream
		{
			this(OutputStream output);
			override IConduit conduit();

			OutputStream output();
			long seek(long ofs, Anchor anchor = Anchor.Begin);
			override void close();

			override size_t write(const(void)[] dst);

			override OutputStream copy(InputStream src, size_t max = -1);

			override OutputStream flush();

			long count();
			private 
			{
				OutputStream sink;
				long _count;
			}
		}
		class SliceSeekInputStream : InputStream
		{
			this(InputStream source, long begin, long length);
			override IConduit conduit();

			override void close();

			override size_t read(void[] dst);

			override void[] load(size_t max = -1);

			override InputStream flush();

			InputStream input();
			override long seek(long offset, Anchor anchor = cast(Anchor)0);

			private 
			{
				InputStream source;
				InputStream seeker;
				long _position;
				long begin;
				long length;
			}
		}
		class SliceInputStream : InputStream
		{
			this(InputStream source, long length);
			override IConduit conduit();

			override void close();

			InputStream input();
			long seek(long ofs, Anchor anchor = Anchor.Begin);
			override size_t read(void[] dst);

			override void[] load(size_t max = -1);

			override InputStream flush();

			private 
			{
				InputStream source;
				long _length;
			}
		}
		class SliceSeekOutputStream : OutputStream
		{
			this(OutputStream source, long begin, long length);
			override IConduit conduit();

			override void close();

			size_t write(const(void)[] src);
			override OutputStream copy(InputStream src, size_t max = -1);

			override OutputStream flush();

			override OutputStream output();

			override long seek(long offset, Anchor anchor = cast(Anchor)0);

			private 
			{
				OutputStream source;
				OutputStream seeker;
				long _position;
				long begin;
				long length;
			}
		}
		class WrapSeekInputStream : InputStream
		{
			this(InputStream source);
			this(InputStream source, long position);
			override IConduit conduit();

			override void close();

			override size_t read(void[] dst);

			override void[] load(size_t max = -1);

			override InputStream flush();

			InputStream input();
			override long seek(long offset, Anchor anchor = cast(Anchor)0);

			private 
			{
				InputStream source;
				InputStream seeker;
				long _position;
			}
		}
		class WrapSeekOutputStream : OutputStream
		{
			this(OutputStream source);
			this(OutputStream source, long position);
			override IConduit conduit();

			override void close();

			size_t write(const(void)[] src);
			override OutputStream copy(InputStream src, size_t max = -1);

			override OutputStream flush();

			override OutputStream output();

			override long seek(long offset, Anchor anchor = cast(Anchor)0);

			private 
			{
				OutputStream source;
				OutputStream seeker;
				long _position;
			}
		}
	}
}
